@@ -119,6 +119,7 @@ class GroupDefImpl : public DefinitionMixin<GroupDef>
     bool hasDetailedDescription() const override;
     void sortSubGroups() override;
     void writeSummaryLinks(OutputList &ol) const override;
+    void writePageNavigation(OutputList &ol) const override;
 
     bool hasGroupGraph() const override;
     void overrideGroupGraph(bool e) override;
@@ -182,7 +183,7 @@ std::unique_ptr<GroupDef> createGroupDef(const QCString &fileName,int line,const
 
 GroupDefImpl::GroupDefImpl(const QCString &df,int dl,const QCString &na,const QCString &t,
                    const QCString &refFileName) : DefinitionMixin(df,dl,1,na),
-                    m_allMemberList(MemberListType_allMembersList,MemberListContainer::Group)
+                    m_allMemberList(MemberListType::AllMembersList(),MemberListContainer::Group)
 {
   if (!refFileName.isEmpty())
   {
@@ -246,7 +247,7 @@ void GroupDefImpl::findSectionsInDocumentation()
 
   for (auto &ml : m_memberLists)
   {
-    if (ml->listType()&MemberListType_declarationLists)
+    if (ml->listType().isDeclaration())
     {
       ml->findSectionsInDocumentation(this);
     }
@@ -347,7 +348,7 @@ void GroupDefImpl::addMembersToMemberGroup()
 {
   for (auto &ml : m_memberLists)
   {
-    if (ml->listType()&MemberListType_declarationLists)
+    if (ml->listType().isDeclaration())
     {
       ::addMembersToMemberGroup(ml.get(),&m_memberGroups,this);
     }
@@ -402,108 +403,108 @@ bool GroupDefImpl::insertMember(MemberDef *md,bool docOnly)
   m_allMemberList.push_back(md);
   switch(md->memberType())
   {
-    case MemberType_Variable:
+    case MemberType::Variable:
       if (!docOnly)
       {
-        addMemberToList(MemberListType_decVarMembers,md);
+        addMemberToList(MemberListType::DecVarMembers(),md);
       }
-      addMemberToList(MemberListType_docVarMembers,md);
+      addMemberToList(MemberListType::DocVarMembers(),md);
       break;
-    case MemberType_Function:
+    case MemberType::Function:
       if (!docOnly)
       {
-        addMemberToList(MemberListType_decFuncMembers,md);
+        addMemberToList(MemberListType::DecFuncMembers(),md);
       }
-      addMemberToList(MemberListType_docFuncMembers,md);
+      addMemberToList(MemberListType::DocFuncMembers(),md);
       break;
-    case MemberType_Typedef:
+    case MemberType::Typedef:
       if (!docOnly)
       {
-        addMemberToList(MemberListType_decTypedefMembers,md);
+        addMemberToList(MemberListType::DecTypedefMembers(),md);
       }
-      addMemberToList(MemberListType_docTypedefMembers,md);
+      addMemberToList(MemberListType::DocTypedefMembers(),md);
       break;
-    case MemberType_Enumeration:
+    case MemberType::Enumeration:
       if (!docOnly)
       {
-        addMemberToList(MemberListType_decEnumMembers,md);
+        addMemberToList(MemberListType::DecEnumMembers(),md);
       }
-      addMemberToList(MemberListType_docEnumMembers,md);
+      addMemberToList(MemberListType::DocEnumMembers(),md);
       break;
-    case MemberType_EnumValue:
+    case MemberType::EnumValue:
       if (!docOnly)
       {
-        addMemberToList(MemberListType_decEnumValMembers,md);
+        addMemberToList(MemberListType::DecEnumValMembers(),md);
       }
-      addMemberToList(MemberListType_docEnumValMembers,md);
+      addMemberToList(MemberListType::DocEnumValMembers(),md);
       break;
-    case MemberType_Define:
+    case MemberType::Define:
       if (!docOnly)
       {
-        addMemberToList(MemberListType_decDefineMembers,md);
+        addMemberToList(MemberListType::DecDefineMembers(),md);
       }
-      addMemberToList(MemberListType_docDefineMembers,md);
+      addMemberToList(MemberListType::DocDefineMembers(),md);
       break;
-    case MemberType_Signal:
+    case MemberType::Signal:
       if (!docOnly)
       {
-        addMemberToList(MemberListType_decSignalMembers,md);
+        addMemberToList(MemberListType::DecSignalMembers(),md);
       }
-      addMemberToList(MemberListType_docSignalMembers,md);
+      addMemberToList(MemberListType::DocSignalMembers(),md);
       break;
-    case MemberType_Slot:
+    case MemberType::Slot:
       if (md->protection()==Protection::Public)
       {
         if (!docOnly)
         {
-          addMemberToList(MemberListType_decPubSlotMembers,md);
+          addMemberToList(MemberListType::DecPubSlotMembers(),md);
         }
-        addMemberToList(MemberListType_docPubSlotMembers,md);
+        addMemberToList(MemberListType::DocPubSlotMembers(),md);
       }
       else if (md->protection()==Protection::Protected)
       {
         if (!docOnly)
         {
-          addMemberToList(MemberListType_decProSlotMembers,md);
+          addMemberToList(MemberListType::DecProSlotMembers(),md);
         }
-        addMemberToList(MemberListType_docProSlotMembers,md);
+        addMemberToList(MemberListType::DocProSlotMembers(),md);
       }
       else
       {
         if (!docOnly)
         {
-          addMemberToList(MemberListType_decPriSlotMembers,md);
+          addMemberToList(MemberListType::DecPriSlotMembers(),md);
         }
-        addMemberToList(MemberListType_docPriSlotMembers,md);
+        addMemberToList(MemberListType::DocPriSlotMembers(),md);
       }
       break;
-    case MemberType_Event:
+    case MemberType::Event:
       if (!docOnly)
       {
-        addMemberToList(MemberListType_decEventMembers,md);
+        addMemberToList(MemberListType::DecEventMembers(),md);
       }
-      addMemberToList(MemberListType_docEventMembers,md);
+      addMemberToList(MemberListType::DocEventMembers(),md);
       break;
-    case MemberType_Property:
+    case MemberType::Property:
       if (!docOnly)
       {
-        addMemberToList(MemberListType_decPropMembers,md);
+        addMemberToList(MemberListType::DecPropMembers(),md);
       }
-      addMemberToList(MemberListType_docPropMembers,md);
+      addMemberToList(MemberListType::DocPropMembers(),md);
       break;
-    case MemberType_Friend:
+    case MemberType::Friend:
       if (!docOnly)
       {
-        addMemberToList(MemberListType_decFriendMembers,md);
+        addMemberToList(MemberListType::DecFriendMembers(),md);
       }
-      addMemberToList(MemberListType_docFriendMembers,md);
+      addMemberToList(MemberListType::DocFriendMembers(),md);
       break;
     default:
       err("GroupDefImpl::insertMembers(): "
-           "member '%s' (typeid=%d) with scope '%s' inserted in group scope '%s'!\n",
-           qPrint(md->name()),md->memberType(),
-           md->getClassDef() ? qPrint(md->getClassDef()->name()) : "",
-           qPrint(name()));
+           "member '{}' (typeid='{}') with scope '{}' inserted in group scope '{}'!\n",
+           md->name(),md->memberTypeName(),
+           md->getClassDef() ? md->getClassDef()->name() : "",
+           name());
   }
   return TRUE;
 }
@@ -516,65 +517,65 @@ void GroupDefImpl::removeMember(MemberDef *md)
   {
     m_allMemberNameInfoLinkedMap.del(md->name());
 
-    removeMemberFromList(MemberListType_allMembersList,md);
+    removeMemberFromList(MemberListType::AllMembersList(),md);
     switch(md->memberType())
     {
-      case MemberType_Variable:
-	removeMemberFromList(MemberListType_decVarMembers,md);
-        removeMemberFromList(MemberListType_docVarMembers,md);
+      case MemberType::Variable:
+	removeMemberFromList(MemberListType::DecVarMembers(),md);
+        removeMemberFromList(MemberListType::DocVarMembers(),md);
         break;
-      case MemberType_Function:
-        removeMemberFromList(MemberListType_decFuncMembers,md);
-        removeMemberFromList(MemberListType_docFuncMembers,md);
+      case MemberType::Function:
+        removeMemberFromList(MemberListType::DecFuncMembers(),md);
+        removeMemberFromList(MemberListType::DocFuncMembers(),md);
         break;
-      case MemberType_Typedef:
-        removeMemberFromList(MemberListType_decTypedefMembers,md);
-        removeMemberFromList(MemberListType_docTypedefMembers,md);
+      case MemberType::Typedef:
+        removeMemberFromList(MemberListType::DecTypedefMembers(),md);
+        removeMemberFromList(MemberListType::DocTypedefMembers(),md);
         break;
-      case MemberType_Enumeration:
-        removeMemberFromList(MemberListType_decEnumMembers,md);
-        removeMemberFromList(MemberListType_docEnumMembers,md);
+      case MemberType::Enumeration:
+        removeMemberFromList(MemberListType::DecEnumMembers(),md);
+        removeMemberFromList(MemberListType::DocEnumMembers(),md);
         break;
-      case MemberType_EnumValue:
-        removeMemberFromList(MemberListType_decEnumValMembers,md);
-        removeMemberFromList(MemberListType_docEnumValMembers,md);
+      case MemberType::EnumValue:
+        removeMemberFromList(MemberListType::DecEnumValMembers(),md);
+        removeMemberFromList(MemberListType::DocEnumValMembers(),md);
         break;
-      case MemberType_Define:
-        removeMemberFromList(MemberListType_decDefineMembers,md);
-        removeMemberFromList(MemberListType_docDefineMembers,md);
+      case MemberType::Define:
+        removeMemberFromList(MemberListType::DecDefineMembers(),md);
+        removeMemberFromList(MemberListType::DocDefineMembers(),md);
         break;
-      case MemberType_Signal:
-        removeMemberFromList(MemberListType_decSignalMembers,md);
-        removeMemberFromList(MemberListType_docSignalMembers,md);
+      case MemberType::Signal:
+        removeMemberFromList(MemberListType::DecSignalMembers(),md);
+        removeMemberFromList(MemberListType::DocSignalMembers(),md);
         break;
-      case MemberType_Slot:
+      case MemberType::Slot:
         if (md->protection()==Protection::Public)
         {
-          removeMemberFromList(MemberListType_decPubSlotMembers,md);
-          removeMemberFromList(MemberListType_docPubSlotMembers,md);
+          removeMemberFromList(MemberListType::DecPubSlotMembers(),md);
+          removeMemberFromList(MemberListType::DocPubSlotMembers(),md);
         }
         else if (md->protection()==Protection::Protected)
         {
-          removeMemberFromList(MemberListType_decProSlotMembers,md);
-          removeMemberFromList(MemberListType_docProSlotMembers,md);
+          removeMemberFromList(MemberListType::DecProSlotMembers(),md);
+          removeMemberFromList(MemberListType::DocProSlotMembers(),md);
         }
         else
         {
-          removeMemberFromList(MemberListType_decPriSlotMembers,md);
-          removeMemberFromList(MemberListType_docPriSlotMembers,md);
+          removeMemberFromList(MemberListType::DecPriSlotMembers(),md);
+          removeMemberFromList(MemberListType::DocPriSlotMembers(),md);
         }
         break;
-      case MemberType_Event:
-        removeMemberFromList(MemberListType_decEventMembers,md);
-        removeMemberFromList(MemberListType_docEventMembers,md);
+      case MemberType::Event:
+        removeMemberFromList(MemberListType::DecEventMembers(),md);
+        removeMemberFromList(MemberListType::DocEventMembers(),md);
         break;
-      case MemberType_Property:
-        removeMemberFromList(MemberListType_decPropMembers,md);
-        removeMemberFromList(MemberListType_docPropMembers,md);
+      case MemberType::Property:
+        removeMemberFromList(MemberListType::DecPropMembers(),md);
+        removeMemberFromList(MemberListType::DocPropMembers(),md);
         break;
-      case MemberType_Friend:
-        removeMemberFromList(MemberListType_decFriendMembers,md);
-        removeMemberFromList(MemberListType_docFriendMembers,md);
+      case MemberType::Friend:
+        removeMemberFromList(MemberListType::DecFriendMembers(),md);
+        removeMemberFromList(MemberListType::DocFriendMembers(),md);
         break;
       default:
         err("GroupDefImpl::removeMember(): unexpected member remove in file!\n");
@@ -800,7 +801,7 @@ void GroupDefImpl::writeDetailedDescription(OutputList &ol,const QCString &title
     {
       ol.disableAllBut(OutputType::Man); // always print title for man page
     }
-    ol.startGroupHeader();
+    ol.startGroupHeader("details");
     ol.parseText(title);
     ol.endGroupHeader();
     ol.popGeneratorState();
@@ -809,7 +810,7 @@ void GroupDefImpl::writeDetailedDescription(OutputList &ol,const QCString &title
     if (!briefDescription().isEmpty() && Config_getBool(REPEAT_BRIEF))
     {
       ol.generateDoc(briefFile(),briefLine(),this,nullptr,briefDescription(),FALSE,FALSE,
-                     QCString(),FALSE,FALSE,Config_getBool(MARKDOWN_SUPPORT));
+                     QCString(),FALSE,FALSE);
     }
     // write separator between brief and details
     if (!briefDescription().isEmpty() && Config_getBool(REPEAT_BRIEF) &&
@@ -829,14 +830,14 @@ void GroupDefImpl::writeDetailedDescription(OutputList &ol,const QCString &title
     if (!documentation().isEmpty())
     {
       ol.generateDoc(docFile(),docLine(),this,nullptr,documentation()+"\n",TRUE,FALSE,
-                     QCString(),FALSE,FALSE,Config_getBool(MARKDOWN_SUPPORT));
+                     QCString(),FALSE,FALSE);
     }
 
     // write inbody documentation
     if (!inbodyDocumentation().isEmpty())
     {
       ol.generateDoc(inbodyFile(),inbodyLine(),this,nullptr,inbodyDocumentation()+"\n",TRUE,FALSE,
-                     QCString(),FALSE,FALSE,Config_getBool(MARKDOWN_SUPPORT));
+                     QCString(),FALSE,FALSE);
     }
   }
 }
@@ -849,7 +850,7 @@ void GroupDefImpl::writeBriefDescription(OutputList &ol)
     auto ast    { validatingParseDoc(*parser.get(),
                                      briefFile(),briefLine(),this,nullptr,
                                      briefDescription(),TRUE,FALSE,
-                                     QCString(),TRUE,FALSE,Config_getBool(MARKDOWN_SUPPORT)) };
+                                     QCString(),TRUE,FALSE) };
     if (!ast->isEmpty())
     {
       ol.startParagraph();
@@ -884,12 +885,12 @@ void GroupDefImpl::writeGroupGraph(OutputList &ol)
     DotGroupCollaboration graph(this);
     if (graph.isTooBig())
     {
-       warn_uncond("Group dependency graph for '%s' not generated, too many nodes (%d), threshold is %d. Consider increasing DOT_GRAPH_MAX_NODES.\n",
-           qPrint(name()), graph.numNodes(), Config_getInt(DOT_GRAPH_MAX_NODES));
+       warn_uncond("Group dependency graph for '{}' not generated, too many nodes ({}), threshold is {}. Consider increasing DOT_GRAPH_MAX_NODES.\n",
+           name(), graph.numNodes(), Config_getInt(DOT_GRAPH_MAX_NODES));
     }
     else if (!graph.isTrivial())
     {
-      msg("Generating dependency graph for group %s\n",qPrint(qualifiedName()));
+      msg("Generating dependency graph for group {}\n",qualifiedName());
       ol.pushGeneratorState();
       ol.disable(OutputType::Man);
       //ol.startParagraph();
@@ -915,7 +916,9 @@ void GroupDefImpl::writeFiles(OutputList &ol,const QCString &title)
     {
       if (!fd->hasDocumentation()) continue;
       ol.startMemberDeclaration();
-      ol.startMemberItem(fd->anchor(),OutputGenerator::MemberItemType::Normal);
+      QCString anc = fd->anchor();
+      if (anc.isEmpty()) anc=fd->displayName(); else anc.prepend(fd->displayName()+"_");
+      ol.startMemberItem(anc,OutputGenerator::MemberItemType::Normal);
       ol.docify(theTranslator->trFile(FALSE,TRUE)+" ");
       ol.insertMemberAlign();
       ol.writeObjectLink(fd->getReference(),fd->getOutputFileBase(),QCString(),fd->displayName());
@@ -924,7 +927,7 @@ void GroupDefImpl::writeFiles(OutputList &ol,const QCString &title)
       {
         ol.startMemberDescription(fd->getOutputFileBase());
         ol.generateDoc(briefFile(),briefLine(),fd,nullptr,fd->briefDescription(),FALSE,FALSE,
-                       QCString(),TRUE,FALSE,Config_getBool(MARKDOWN_SUPPORT));
+                       QCString(),TRUE,FALSE);
         ol.endMemberDescription();
       }
       ol.endMemberDeclaration(QCString(),QCString());
@@ -959,9 +962,9 @@ void GroupDefImpl::writeNestedGroups(OutputList &ol,const QCString &title)
       {
         if (!gd->hasDocumentation()) continue;
         ol.startMemberDeclaration();
-        ol.startMemberItem(gd->anchor(),OutputGenerator::MemberItemType::Normal);
-        //ol.docify(theTranslator->trGroup(FALSE,TRUE));
-        //ol.docify(" ");
+        QCString anc = gd->anchor();
+        if (anc.isEmpty()) anc=gd->name(); else anc.prepend(gd->name()+"_");
+        ol.startMemberItem(anc,OutputGenerator::MemberItemType::Normal);
         ol.insertMemberAlign();
         ol.writeObjectLink(gd->getReference(),gd->getOutputFileBase(),QCString(),gd->groupTitle());
         ol.endMemberItem(OutputGenerator::MemberItemType::Normal);
@@ -969,7 +972,7 @@ void GroupDefImpl::writeNestedGroups(OutputList &ol,const QCString &title)
         {
           ol.startMemberDescription(gd->getOutputFileBase());
           ol.generateDoc(briefFile(),briefLine(),gd,nullptr,gd->briefDescription(),FALSE,FALSE,
-                         QCString(),TRUE,FALSE,Config_getBool(MARKDOWN_SUPPORT));
+                         QCString(),TRUE,FALSE);
           ol.endMemberDescription();
         }
         ol.endMemberDeclaration(QCString(),QCString());
@@ -992,7 +995,9 @@ void GroupDefImpl::writeDirs(OutputList &ol,const QCString &title)
     {
       if (!dd->hasDocumentation()) continue;
       ol.startMemberDeclaration();
-      ol.startMemberItem(dd->anchor(),OutputGenerator::MemberItemType::Normal);
+      QCString anc = dd->anchor();
+      if (anc.isEmpty()) anc=dd->shortName(); else anc.prepend(dd->shortName()+"_");
+      ol.startMemberItem(anc,OutputGenerator::MemberItemType::Normal);
       ol.parseText(theTranslator->trDir(FALSE,TRUE));
       ol.insertMemberAlign();
       ol.writeObjectLink(dd->getReference(),dd->getOutputFileBase(),QCString(),dd->shortName());
@@ -1001,7 +1006,7 @@ void GroupDefImpl::writeDirs(OutputList &ol,const QCString &title)
       {
         ol.startMemberDescription(dd->getOutputFileBase());
         ol.generateDoc(briefFile(),briefLine(),dd,nullptr,dd->briefDescription(),FALSE,FALSE,
-                       QCString(),TRUE,FALSE,Config_getBool(MARKDOWN_SUPPORT));
+                       QCString(),TRUE,FALSE);
         ol.endMemberDescription();
       }
       ol.endMemberDeclaration(QCString(),QCString());
@@ -1051,7 +1056,7 @@ void GroupDefImpl::writePageDocumentation(OutputList &ol)
       }
       ol.startTextBlock();
       ol.generateDoc(pd->docFile(),pd->docLine(),pd,nullptr,(pd->documentation()+pd->inbodyDocumentation()),TRUE,FALSE,
-                     QCString(),TRUE,FALSE,Config_getBool(MARKDOWN_SUPPORT));
+                     QCString(),TRUE,FALSE);
       ol.endTextBlock();
     }
   }
@@ -1148,7 +1153,7 @@ void GroupDefImpl::writeSummaryLinks(OutputList &ol) const
         MemberList * ml = getMemberList(lmd->type);
         if (ml && ml->declVisible())
         {
-          ol.writeSummaryLink(QCString(),MemberList::listTypeAsString(ml->listType()),lmd->title(lang),first);
+          ol.writeSummaryLink(QCString(),ml->listType().toLabel(),lmd->title(lang),first);
           first=FALSE;
         }
       }
@@ -1161,9 +1166,14 @@ void GroupDefImpl::writeSummaryLinks(OutputList &ol) const
   ol.popGeneratorState();
 }
 
+void GroupDefImpl::writePageNavigation(OutputList &ol) const
+{
+  ol.writePageOutline();
+}
+
 void GroupDefImpl::writeDocumentation(OutputList &ol)
 {
-  //bool generateTreeView = Config_getBool(GENERATE_TREEVIEW);
+  bool generateTreeView = Config_getBool(GENERATE_TREEVIEW);
   ol.pushGeneratorState();
 
   // Find out how deep this group is nested. In case of multiple parents, use the first one.
@@ -1179,17 +1189,24 @@ void GroupDefImpl::writeDocumentation(OutputList &ol)
             FALSE /* additionalIndices*/, QCString() /*altSidebarName*/, hierarchyLevel);
 
   ol.startHeaderSection();
-  writeSummaryLinks(ol);
+  bool writeOutlinePanel = generateTreeView && Config_getBool(PAGE_OUTLINE_PANEL);
+  if (!writeOutlinePanel) writeSummaryLinks(ol);
   ol.startTitleHead(getOutputFileBase());
+  //1.{
   ol.pushGeneratorState();
   ol.disable(OutputType::Man);
-  ol.parseText(m_title);
+  ol.generateDoc(docFile(),getStartBodyLine(),this,nullptr,m_title,TRUE,FALSE,
+                 QCString(),TRUE,FALSE,Config_getBool(MARKDOWN_SUPPORT),false);
   ol.popGeneratorState();
+  //1.}
   addGroupListToTitle(ol,this);
+  //2.{
   ol.pushGeneratorState();
   ol.disable(OutputType::Man);
   ol.endTitleHead(getOutputFileBase(),m_title);
   ol.popGeneratorState();
+  //2.}
+  //3.{
   ol.pushGeneratorState();
   ol.disableAllBut(OutputType::Man);
   ol.endTitleHead(getOutputFileBase(),name());
@@ -1199,6 +1216,7 @@ void GroupDefImpl::writeDocumentation(OutputList &ol)
     ol.parseText(m_title);
   }
   ol.popGeneratorState();
+  //3.}
   ol.endHeaderSection();
   ol.startContents();
 
@@ -1317,8 +1335,8 @@ void GroupDefImpl::writeDocumentation(OutputList &ol)
       case LayoutDocEntry::DirSubDirs:
       case LayoutDocEntry::DirFiles:
       case LayoutDocEntry::DirGraph:
-        err("Internal inconsistency: member %d should not be part of "
-            "LayoutDocManager::Group entry list\n",lde->kind());
+        err("Internal inconsistency: member '{}' should not be part of "
+            "LayoutDocManager::Group entry list\n",lde->entryToString());
         break;
     }
   }
@@ -1340,7 +1358,21 @@ void GroupDefImpl::writeDocumentation(OutputList &ol)
       }
     }
   }
-  endFile(ol);
+  if (generateTreeView && Config_getBool(PAGE_OUTLINE_PANEL))
+  {
+    ol.pushGeneratorState();
+    ol.disableAllBut(OutputType::Html);
+    ol.endContents();
+    ol.writeString("</div><!-- doc-content -->\n");
+    writePageNavigation(ol);
+    ol.writeString("</div><!-- container -->\n");
+    ol.popGeneratorState();
+    endFile(ol,true,true);
+  }
+  else
+  {
+    endFile(ol);
+  }
 
   ol.popGeneratorState();
 
@@ -1358,7 +1390,7 @@ void GroupDefImpl::writeMemberPages(OutputList &ol, int hierarchyLevel)
 
   for (const auto &ml : m_memberLists)
   {
-    if (ml->listType()&MemberListType_documentationLists)
+    if (ml->listType().isDocumentation())
     {
        ml->writeDocumentationPage(ol,name(),this,hierarchyLevel);
     }
@@ -1390,13 +1422,13 @@ void GroupDefImpl::writeQuickMemberLinks(OutputList &ol,const MemberDef *current
         {
           ol.writeString("          <tr><td class=\"navtab\">");
         }
-        ol.writeString("<a class=\"navtab\" ");
+        ol.writeString("<span class=\"label\"><a ");
         ol.writeString("href=\"");
         if (createSubDirs) ol.writeString("../../");
         ol.writeString(fn+"#"+md->anchor());
         ol.writeString("\">");
         ol.writeString(convertToHtml(md->localName()));
-        ol.writeString("</a>");
+        ol.writeString("</a></span>");
         ol.writeString("</td></tr>\n");
       }
     }
@@ -1427,8 +1459,8 @@ void addClassToGroups(const Entry *root,ClassDef *cd)
     else if (!gd && g.pri == Grouping::GROUPING_INGROUP)
     {
       warn(root->fileName, root->startLine,
-          "Found non-existing group '%s' for the command '%s', ignoring command",
-          qPrint(g.groupname), Grouping::getGroupPriName( g.pri )
+          "Found non-existing group '{}' for the command '{}', ignoring command",
+          g.groupname, Grouping::getGroupPriName( g.pri )
           );
     }
   }
@@ -1451,8 +1483,8 @@ void addConceptToGroups(const Entry *root,ConceptDef *cd)
     else if (!gd && g.pri == Grouping::GROUPING_INGROUP)
     {
       warn(root->fileName, root->startLine,
-          "Found non-existing group '%s' for the command '%s', ignoring command",
-          qPrint(g.groupname), Grouping::getGroupPriName( g.pri )
+          "Found non-existing group '{}' for the command '{}', ignoring command",
+          g.groupname, Grouping::getGroupPriName( g.pri )
           );
     }
   }
@@ -1471,8 +1503,8 @@ void addModuleToGroups(const Entry *root,ModuleDef *mod)
     else if (!gd && g.pri == Grouping::GROUPING_INGROUP)
     {
       warn(root->fileName, root->startLine,
-          "Found non-existing group '%s' for the command '%s', ignoring command",
-          qPrint(g.groupname), Grouping::getGroupPriName( g.pri )
+          "Found non-existing group '{}' for the command '{}', ignoring command",
+          g.groupname, Grouping::getGroupPriName( g.pri )
           );
     }
   }
@@ -1499,8 +1531,8 @@ void addNamespaceToGroups(const Entry *root,NamespaceDef *nd)
     else if (!gd && g.pri == Grouping::GROUPING_INGROUP)
     {
       warn(root->fileName, root->startLine,
-          "Found non-existing group '%s' for the command '%s', ignoring command",
-          qPrint(g.groupname), Grouping::getGroupPriName( g.pri )
+          "Found non-existing group '{}' for the command '{}', ignoring command",
+          g.groupname, Grouping::getGroupPriName( g.pri )
           );
     }
   }
@@ -1522,8 +1554,8 @@ void addDirToGroups(const Entry *root,DirDef *dd)
     else if (!gd && g.pri == Grouping::GROUPING_INGROUP)
     {
       warn(root->fileName, root->startLine,
-          "Found non-existing group '%s' for the command '%s', ignoring command",
-          qPrint(g.groupname), Grouping::getGroupPriName( g.pri )
+          "Found non-existing group '{}' for the command '{}', ignoring command",
+          g.groupname, Grouping::getGroupPriName( g.pri )
           );
     }
   }
@@ -1539,13 +1571,13 @@ void addGroupToGroups(const Entry *root,GroupDef *subGroup)
     {
       if (gd==subGroup)
       {
-        warn(root->fileName,root->startLine,"Refusing to add group %s to itself",
-            qPrint(gd->name()));
+        warn(root->fileName,root->startLine,"Refusing to add group {} to itself",
+            gd->name());
       }
       else if (subGroup->findGroup(gd))
       {
-        warn(root->fileName,root->startLine,"Refusing to add group %s to group %s, since the latter is already a "
-                                            "subgroup of the former", qPrint(subGroup->name()),qPrint(gd->name()));
+        warn(root->fileName,root->startLine,"Refusing to add group {} to group {}, since the latter is already a "
+                                            "subgroup of the former", subGroup->name(),gd->name());
       }
       else if (!gd->findGroup(subGroup))
       {
@@ -1556,8 +1588,8 @@ void addGroupToGroups(const Entry *root,GroupDef *subGroup)
     else if (!gd && g.pri == Grouping::GROUPING_INGROUP)
     {
       warn(root->fileName, root->startLine,
-          "Found non-existing group '%s' for the command '%s', ignoring command",
-          qPrint(g.groupname), Grouping::getGroupPriName( g.pri )
+          "Found non-existing group '{}' for the command '{}', ignoring command",
+          g.groupname, Grouping::getGroupPriName( g.pri )
           );
     }
   }
@@ -1581,10 +1613,10 @@ void addMemberToGroups(const Entry *root,MemberDef *md)
       if (fgd && gd!=fgd && g.pri==pri)
       {
         warn(root->fileName, root->startLine,
-            "Member %s found in multiple %s groups! "
-            "The member will be put in group %s, and not in group %s",
-            qPrint(md->name()), Grouping::getGroupPriName( pri ),
-            qPrint(gd->name()), qPrint(fgd->name())
+            "Member {} found in multiple {} groups! "
+            "The member will be put in group {}, and not in group {}",
+            md->name(), Grouping::getGroupPriName( pri ),
+            gd->name(), fgd->name()
             );
       }
 
@@ -1594,8 +1626,8 @@ void addMemberToGroups(const Entry *root,MemberDef *md)
     else if (!gd && g.pri == Grouping::GROUPING_INGROUP)
     {
       warn(root->fileName, root->startLine,
-          "Found non-existing group '%s' for the command '%s', ignoring command",
-          qPrint(g.groupname), Grouping::getGroupPriName( g.pri )
+          "Found non-existing group '{}' for the command '{}', ignoring command",
+          g.groupname, Grouping::getGroupPriName( g.pri )
           );
     }
   }
@@ -1604,7 +1636,7 @@ void addMemberToGroups(const Entry *root,MemberDef *md)
   // put member into group defined by this entry?
   if (fgd)
   {
-    GroupDef *mgd = const_cast<GroupDef*>(md->getGroupDef());
+    GroupDef *mgd = md->getGroupDef();
     //printf("mgd=%p\n",mgd);
     bool insertit = FALSE;
     if (mgd==nullptr)
@@ -1633,12 +1665,11 @@ void addMemberToGroups(const Entry *root,MemberDef *md)
           else if (!root->doc.isEmpty() && md->getGroupHasDocs())
           {
             warn(md->getGroupFileName(),md->getGroupStartLine(),
-                "Member documentation for %s found several times in %s groups!\n"
-                "%s:%d: The member will remain in group %s, and won't be put into group %s",
-                qPrint(md->name()), Grouping::getGroupPriName( pri ),
-                qPrint(root->fileName), root->startLine,
-                qPrint(mgd->name()),
-                qPrint(fgd->name())
+                "Member documentation for {} found several times in {} groups!\n"
+                "{}:{}: The member will remain in group {}, and won't be put into group {}",
+                md->name(), Grouping::getGroupPriName( pri ),
+                root->fileName, root->startLine,
+                mgd->name(), fgd->name()
                 );
           }
         }
@@ -1704,8 +1735,8 @@ void addExampleToGroups(const Entry *root,PageDef *eg)
     else if (!gd && g.pri == Grouping::GROUPING_INGROUP)
     {
       warn(root->fileName, root->startLine,
-          "Found non-existing group '%s' for the command '%s', ignoring command",
-          qPrint(g.groupname), Grouping::getGroupPriName( g.pri )
+          "Found non-existing group '{}' for the command '{}', ignoring command",
+          g.groupname, Grouping::getGroupPriName( g.pri )
           );
     }
   }
@@ -1734,7 +1765,7 @@ void GroupDefImpl::addListReferences()
   }
   for (auto &ml : m_memberLists)
   {
-    if (ml->listType()&MemberListType_documentationLists)
+    if (ml->listType().isDocumentation())
     {
       ml->addListReferences(this);
     }
@@ -1747,8 +1778,8 @@ void GroupDefImpl::addMemberToList(MemberListType lt,MemberDef *md)
   bool sortMemberDocs = Config_getBool(SORT_MEMBER_DOCS);
   const auto &ml = m_memberLists.get(lt,MemberListContainer::Group);
   ml->setNeedsSorting(
-      ((ml->listType()&MemberListType_declarationLists) && sortBriefDocs) ||
-      ((ml->listType()&MemberListType_documentationLists) && sortMemberDocs));
+      (ml->listType().isDeclaration() && sortBriefDocs) ||
+      (ml->listType().isDocumentation() && sortMemberDocs));
   ml->push_back(md);
 }
 
@@ -1875,7 +1906,7 @@ void GroupDefImpl::writeMemberDeclarations(OutputList &ol,MemberListType lt,cons
 void GroupDefImpl::writeMemberDocumentation(OutputList &ol,MemberListType lt,const QCString &title)
 {
   MemberList * ml = getMemberList(lt);
-  if (ml) ml->writeDocumentation(ol,name(),this,title);
+  if (ml) ml->writeDocumentation(ol,name(),this,title,ml->listType().toLabel());
 }
 
 void GroupDefImpl::removeMemberFromList(MemberListType lt,MemberDef *md)
@@ -1896,7 +1927,7 @@ static bool hasNonReferenceNestedGroupRec(const GroupDef *gd,int level)
 {
   if (level>30)
   {
-    err("Possible recursive group relation while inside %s\n",qPrint(gd->name()));
+    err("Possible recursive group relation while inside {}\n",gd->name());
     return false;
   }
   bool found=gd->isLinkableInProject();
