@@ -34,6 +34,10 @@ class LatexCodeGenerator : public OutputCodeIntf
     OutputType type() const override { return OutputType::Latex; }
     std::unique_ptr<OutputCodeIntf> clone() override { return std::make_unique<LatexCodeGenerator>(*this); }
     void codify(const QCString &text) override;
+    void stripCodeComments(bool b) override;
+    void startSpecialComment() override;
+    void endSpecialComment() override;
+    void setStripIndentAmount(size_t amount) override;
     void writeCodeLink(CodeSymbolType type,
                        const QCString &ref,const QCString &file,
                        const QCString &anchor,const QCString &name,
@@ -80,6 +84,9 @@ class LatexCodeGenerator : public OutputCodeIntf
     bool m_doxyCodeLineOpen = false;
     int m_usedTableLevel = 0;
     bool m_insideTabbing = false;
+    bool m_stripCodeComments = false;
+    bool m_hide = false;
+    size_t m_stripIndentAmount = 0;
 };
 
 /** Generator for LaTeX output. */
@@ -149,7 +156,7 @@ class LatexGenerator : public OutputGenerator, public OutputGenIntf
     void endTextLink() override;
     void startTypewriter() override { m_t << "{\\ttfamily "; }
     void endTypewriter() override { m_t << "}";      }
-    void startGroupHeader(int) override;
+    void startGroupHeader(const QCString &,int) override;
     void endGroupHeader(int) override;
     void startItemListItem() override { m_t << "\\item \n"; }
     void endItemListItem() override {}
@@ -177,7 +184,7 @@ class LatexGenerator : public OutputGenerator, public OutputGenIntf
     void startCompoundTemplateParams() override { m_t << "\\subsubsection*{";}
     void endCompoundTemplateParams() override { m_t << "}\n"; }
 
-    void startMemberGroupHeader(bool) override;
+    void startMemberGroupHeader(const QCString &,bool) override;
     void endMemberGroupHeader() override;
     void startMemberGroupDocs() override;
     void endMemberGroupDocs() override;
@@ -229,11 +236,12 @@ class LatexGenerator : public OutputGenerator, public OutputGenIntf
     void endPageRef(const QCString &,const QCString &) override;
     void startQuickIndices() override {}
     void endQuickIndices() override {}
-    void writeSplitBar(const QCString &) override {}
+    void writeSplitBar(const QCString &,const QCString &) override {}
     void writeNavigationPath(const QCString &) override {}
     void writeLogo() override {}
-    void writeQuickLinks(HighlightedItem,const QCString &) override {}
+    void writeQuickLinks(HighlightedItem,const QCString &,bool) override {}
     void writeSummaryLink(const QCString &,const QCString &,const QCString &,bool) override {}
+    void writePageOutline() override {}
     void startContents() override {}
     void endContents() override {}
     void writeNonBreakableSpace(int) override;
@@ -303,7 +311,11 @@ class LatexGenerator : public OutputGenerator, public OutputGenIntf
     void writeLabel(const QCString &l,bool isLast) override;
     void endLabels() override;
 
-    void writeLocalToc(const SectionRefs &sr,const LocalToc &lt) override;
+    void startLocalToc(int level) override;
+    void endLocalToc() override {}
+    void startTocEntry(const SectionInfo *) override {}
+    void endTocEntry(const SectionInfo *) override {}
+
     void startPlainFile(const QCString &name) override { OutputGenerator::startPlainFile(name); }
     void endPlainFile() override { OutputGenerator::endPlainFile(); }
 
