@@ -28,6 +28,7 @@
 #include "emoji.h"
 #include "fileinfo.h"
 #include "codefragment.h"
+#include "cite.h"
 
 ManDocVisitor::ManDocVisitor(TextStream &t,OutputCodeList &ci,
                              const QCString &langExt)
@@ -82,7 +83,7 @@ void ManDocVisitor::operator()(const DocSymbol &s)
   else
   {
     // no error or warning to be supplied
-    // err("man: non supported HTML-entity found: &%s;\n",get_symbol_item(s->symbol()));
+    // err("man: non supported HTML-entity found: &{};\n",get_symbol_item(s->symbol()));
   }
   m_firstCol=FALSE;
 }
@@ -148,6 +149,7 @@ void ManDocVisitor::operator()(const DocStyleChange &s)
       m_firstCol=FALSE;
       break;
     case DocStyleChange::Kbd:
+    case DocStyleChange::Typewriter:
     case DocStyleChange::Code:
       if (s.enable()) m_t << "\\fR";   else m_t << "\\fP";
       m_firstCol=FALSE;
@@ -216,18 +218,18 @@ void ManDocVisitor::operator()(const DocVerbatim &s)
       m_firstCol=TRUE;
       break;
     case DocVerbatim::JavaDocLiteral:
-      filter(s.text());
+      filter(s.text(),true);
       break;
     case DocVerbatim::JavaDocCode:
       m_t << "\\fR\n";
-      filter(s.text());
+      filter(s.text(),true);
       m_t << "\\fP\n";
       break;
     case DocVerbatim::Verbatim:
       if (!m_firstCol) m_t << "\n";
       m_t << ".PP\n";
       m_t << ".nf\n";
-      filter(s.text());
+      filter(s.text(),true);
       if (!m_firstCol) m_t << "\n";
       m_t << ".fi\n";
       m_t << ".PP\n";
@@ -426,9 +428,19 @@ void ManDocVisitor::operator()(const DocCite &cite)
 {
   if (m_hide) return;
   m_t << "\\fB";
-  if (cite.file().isEmpty()) m_t << "[";
-  filter(cite.text());
-  if (cite.file().isEmpty()) m_t << "]";
+  auto opt = cite.option();
+  QCString txt;
+  if (!cite.file().isEmpty())
+  {
+    txt = cite.getText();
+  }
+  else
+  {
+    if (!opt.noPar()) txt += "[";
+    txt += cite.target();
+    if (!opt.noPar()) txt += "]";
+  }
+  filter(txt);
   m_t << "\\fP";
 }
 
@@ -984,18 +996,21 @@ void ManDocVisitor::operator()(const DocParBlock &pb)
   visitChildren(pb);
 }
 
-void ManDocVisitor::filter(const QCString &str)
+void ManDocVisitor::filter(const QCString &str, const bool retainNewline)
 {
   if (!str.isEmpty())
   {
     const char *p=str.data();
     char c=0;
+    bool insideDoubleQuote = false;
     while ((c=*p++))
     {
       switch(c)
       {
         case '.':  m_t << "\\&."; break; // see  bug652277
         case '\\': m_t << "\\\\"; break;
+        case '\"': m_t << "\""; insideDoubleQuote = !insideDoubleQuote; break;
+        case '\n': if (retainNewline || !insideDoubleQuote) m_t << c; break;
         default: m_t << c; break;
       }
     }
