@@ -276,12 +276,12 @@ void ManGenerator::init()
   Dir d(manOutput.str());
   if (!d.exists() && !d.mkdir(manOutput.str()))
   {
-    term("Could not create output directory %s\n",qPrint(manOutput));
+    term("Could not create output directory {}\n",manOutput);
   }
   std::string manDir = manOutput.str()+"/"+getSubdir().str();
   if (!d.exists(manDir) && !d.mkdir(manDir))
   {
-    term("Could not create output directory %s/%s\n",qPrint(manOutput), qPrint(getSubdir()));
+    term("Could not create output directory {}/{}\n",manOutput,getSubdir());
   }
   createSubDirs(d);
 }
@@ -427,7 +427,7 @@ void ManGenerator::writeObjectLink(const QCString &,const QCString &,
   startBold(); docify(name); endBold();
 }
 
-void ManGenerator::startGroupHeader(int)
+void ManGenerator::startGroupHeader(const QCString &,int)
 {
   if (!m_firstCol) m_t << "\n";
   m_t << ".SH \"";
@@ -648,7 +648,7 @@ void ManGenerator::endMemberList()
   }
 }
 
-void ManGenerator::startMemberGroupHeader(bool)
+void ManGenerator::startMemberGroupHeader(const QCString &,bool)
 {
   m_t << "\n.PP\n.RI \"\\fB";
 }
@@ -686,7 +686,7 @@ void ManGenerator::startSection(const QCString &,const QCString &,SectionType ty
     switch(type.level())
     {
       case SectionType::Page:             // fall through
-      case SectionType::Section:          startGroupHeader(0); break;
+      case SectionType::Section:          startGroupHeader("",0); break;
       case SectionType::Subsection:       // fall through
       case SectionType::Subsubsection:    // fall through
       case SectionType::Paragraph:        // fall through
