@@ -67,6 +67,7 @@ class ModuleDefImpl : public DefinitionMixin<ModuleDef>
       else return hasDocumentation(); }
     QCString qualifiedName() const override;
     void writeSummaryLinks(OutputList &ol) const override;
+    void writePageNavigation(OutputList &ol) const override;
 
     // --- ModuleDef
     Type moduleType() const override { return m_type; }
@@ -339,15 +340,25 @@ void ModuleDefImpl::mergeSymbolsFrom(ModuleDefImpl *other)
 void ModuleDefImpl::writeDocumentation(OutputList &ol)
 {
   if (isReference()) return;
+  bool generateTreeView = Config_getBool(GENERATE_TREEVIEW);
   ol.pushGeneratorState();
   AUTO_TRACE("%s file=%s",name(),getDefFileName());
   SrcLangExt lang = getLanguage();
-  QCString pageTitle = theTranslator->trModuleReference(displayName());
+  QCString pageTitle;
+  if (Config_getBool(HIDE_COMPOUND_REFERENCE))
+  {
+    pageTitle = displayName();
+  }
+  else
+  {
+    pageTitle = theTranslator->trModuleReference(displayName());
+  }
   startFile(ol,getOutputFileBase(),name(),pageTitle,HighlightedItem::ModuleVisible,false,QCString(),0);
 
   // ---- title part
   ol.startHeaderSection();
-  writeSummaryLinks(ol);
+  bool writeOutlinePanel = generateTreeView && Config_getBool(PAGE_OUTLINE_PANEL);
+  if (!writeOutlinePanel) writeSummaryLinks(ol);
   ol.startTitleHead(getOutputFileBase());
 
   ol.pushGeneratorState();
@@ -467,14 +478,24 @@ void ModuleDefImpl::writeDocumentation(OutputList &ol)
       case LayoutDocEntry::DirSubDirs:
       case LayoutDocEntry::DirFiles:
       case LayoutDocEntry::DirGraph:
-        err("Internal inconsistency: member '%s' should not be part of "
-            "LayoutDocManager::Module entry list\n",qPrint(lde->entryToString()));
+        err("Internal inconsistency: member '{}' should not be part of "
+            "LayoutDocManager::Module entry list\n",lde->entryToString());
         break;
     }
   }
 
   //---------------------------------------- end flexible part -------------------------------
-  endFile(ol);
+  if (generateTreeView)
+  {
+    ol.pushGeneratorState();
+    ol.disableAllBut(OutputType::Html);
+    ol.endContents();
+    ol.writeString("</div><!-- doc-content -->\n");
+    writePageNavigation(ol);
+    ol.writeString("</div><!-- container -->\n");
+    ol.popGeneratorState();
+  }
+  endFile(ol,generateTreeView,true);
 
   ol.popGeneratorState();
 }
@@ -529,7 +550,7 @@ void ModuleDefImpl::writeDetailedDescription(OutputList &ol,const QCString &titl
       ol.disableAllBut(OutputType::Html);
       ol.writeAnchor(QCString(),"details");
     ol.popGeneratorState();
-    ol.startGroupHeader();
+    ol.startGroupHeader("details");
     ol.parseText(title);
     ol.endGroupHeader();
 
@@ -537,7 +558,7 @@ void ModuleDefImpl::writeDetailedDescription(OutputList &ol,const QCString &titl
     if (!briefDescription().isEmpty() && Config_getBool(REPEAT_BRIEF))
     {
       ol.generateDoc(briefFile(),briefLine(),this,nullptr,briefDescription(),FALSE,FALSE,
-                     QCString(),FALSE,FALSE,Config_getBool(MARKDOWN_SUPPORT));
+                     QCString(),FALSE,FALSE);
     }
     if (!briefDescription().isEmpty() && Config_getBool(REPEAT_BRIEF) &&
         !documentation().isEmpty())
@@ -554,7 +575,7 @@ void ModuleDefImpl::writeDetailedDescription(OutputList &ol,const QCString &titl
     if (!documentation().isEmpty())
     {
       ol.generateDoc(docFile(),docLine(),this,nullptr,documentation()+"\n",TRUE,FALSE,
-                     QCString(),FALSE,FALSE,Config_getBool(MARKDOWN_SUPPORT));
+                     QCString(),FALSE,FALSE);
     }
     ol.endTextBlock();
   }
@@ -568,7 +589,7 @@ void ModuleDefImpl::writeBriefDescription(OutputList &ol)
     auto ast    { validatingParseDoc(*parser.get(),
                                      briefFile(),briefLine(),this,nullptr,
                                      briefDescription(),TRUE,FALSE,
-                                     QCString(),TRUE,FALSE,Config_getBool(MARKDOWN_SUPPORT)) };
+                                     QCString(),TRUE,FALSE) };
     if (!ast->isEmpty())
     {
       ol.startParagraph();
@@ -613,7 +634,7 @@ void ModuleDefImpl::writeMemberDeclarations(OutputList &ol,MemberListType lt,con
 void ModuleDefImpl::writeMemberDocumentation(OutputList &ol,MemberListType lt,const QCString &title)
 {
   MemberList * ml = getMemberList(lt);
-  if (ml) ml->writeDocumentation(ol,name(),this,title);
+  if (ml) ml->writeDocumentation(ol,name(),this,title,ml->listType().toLabel());
 }
 
 void ModuleDefImpl::writeAuthorSection(OutputList &ol)
@@ -822,6 +843,11 @@ void ModuleDefImpl::writeSummaryLinks(OutputList &ol) const
   ol.popGeneratorState();
 }
 
+void ModuleDefImpl::writePageNavigation(OutputList &ol) const
+{
+  ol.writePageOutline();
+}
+
 void ModuleDefImpl::writeDeclarationLink(OutputList &ol,bool &found,const QCString &header,bool localNames) const
 {
   if (isLinkable())
@@ -842,9 +868,11 @@ void ModuleDefImpl::writeDeclarationLink(OutputList &ol,bool &found,const QCStri
       found=TRUE;
     }
     ol.startMemberDeclaration();
-    ol.startMemberItem(anchor(),OutputGenerator::MemberItemType::Normal);
-    ol.writeString("module ");
     QCString cname = displayName(!localNames);
+    QCString anc = anchor();
+    if (anc.isEmpty()) anc=cname; else anc.prepend(cname+"_");
+    ol.startMemberItem(anc,OutputGenerator::MemberItemType::Normal);
+    ol.writeString("module ");
     ol.insertMemberAlign();
     if (isLinkable())
     {
@@ -914,7 +942,7 @@ void ModuleDefImpl::writeExports(OutputList &ol,const QCString &title)
         {
           ol.startMemberDescription(mod->getOutputFileBase());
           ol.generateDoc(briefFile(),briefLine(),mod,nullptr,mod->briefDescription(),FALSE,FALSE,
-              QCString(),TRUE,FALSE,Config_getBool(MARKDOWN_SUPPORT));
+              QCString(),TRUE,FALSE);
           ol.endMemberDescription();
         }
         ol.endMemberDeclaration(QCString(),QCString());
@@ -939,7 +967,10 @@ void ModuleDefImpl::writeFiles(OutputList &ol,const QCString &title)
       if (fd)
       {
         ol.startMemberDeclaration();
-        ol.startMemberItem(fd->anchor(),OutputGenerator::MemberItemType::Normal);
+        QCString fname = fd->displayName();
+        QCString anc = fd->anchor();
+        if (anc.isEmpty()) anc=fname; else anc.prepend(fname+"_");
+        ol.startMemberItem(anc,OutputGenerator::MemberItemType::Normal);
         ol.docify(theTranslator->trFile(FALSE,TRUE)+" ");
         ol.insertMemberAlign();
         QCString path=fd->getPath();
@@ -949,7 +980,7 @@ void ModuleDefImpl::writeFiles(OutputList &ol,const QCString &title)
         }
         if (fd->isLinkable())
         {
-          ol.writeObjectLink(fd->getReference(),fd->getOutputFileBase(),QCString(),fd->displayName());
+          ol.writeObjectLink(fd->getReference(),fd->getOutputFileBase(),QCString(),fname);
         }
         else
         {
@@ -962,7 +993,7 @@ void ModuleDefImpl::writeFiles(OutputList &ol,const QCString &title)
         {
           ol.startMemberDescription(fd->getOutputFileBase());
           ol.generateDoc(briefFile(),briefLine(),fd,nullptr,fd->briefDescription(),FALSE,FALSE,
-              QCString(),TRUE,FALSE,Config_getBool(MARKDOWN_SUPPORT));
+              QCString(),TRUE,FALSE);
           ol.endMemberDescription();
         }
         ol.endMemberDeclaration(QCString(),QCString());
@@ -1523,8 +1554,8 @@ void ModuleManager::addDocs(const Entry *root)
   if (root->doc.isEmpty() && root->brief.isEmpty()) return;
   if (root->name.find(':')!=-1)
   {
-    warn(root->fileName,root->startLine,"Ignoring documentation for module partition %s. Please place documentation at the primary module name",
-        qPrint(root->name));
+    warn(root->fileName,root->startLine,"Ignoring documentation for module partition {}. Please place documentation at the primary module name",
+        root->name);
   }
   else
   {
@@ -1545,12 +1576,12 @@ void ModuleManager::addDocs(const Entry *root)
       }
       else
       {
-        warn(root->fileName,root->startLine,"Found documentation for module %s but it has no primary interface unit.",qPrint(root->name));
+        warn(root->fileName,root->startLine,"Found documentation for module {} but it has no primary interface unit.",root->name);
       }
     }
     else
     {
-      warn(root->fileName,root->startLine,"Found documentation for unknown module %s.",qPrint(root->name));
+      warn(root->fileName,root->startLine,"Found documentation for unknown module {}.",root->name);
     }
   }
 }
