@@ -170,7 +170,7 @@ class Ex::Private
 {
   public:
     /** Creates the private part */
-    Private(const std::string &pat) : pattern(pat)
+    Private(std::string_view pat) : pattern(pat)
     {
       data.reserve(100);
     }
@@ -178,7 +178,8 @@ class Ex::Private
 #if ENABLE_DEBUG
     void dump();
 #endif
-    bool matchAt(size_t tokenPos,const std::string &str,Match &match,size_t pos,int level) const;
+    bool matchAt(size_t tokenPos,size_t tokenLen,std::string_view str,
+                 Match &match,size_t pos,int level) const;
 
     /** Flag indicating the expression was successfully compiled */
     bool error = false;
@@ -200,7 +201,7 @@ void Ex::Private::compile()
   if (pattern.empty()) return;
   const char *start = pattern.c_str();
   const char *ps = start;
-  char c;
+  char c = 0;
 
   int prevTokenPos=-1;
   int tokenPos=0;
@@ -377,6 +378,14 @@ void Ex::Private::compile()
             prevTokenPos+=ddiff;
             tokenPos+=ddiff;
           }
+          if (data[prevTokenPos].kind()==PToken::Kind::EndCapture)
+          {
+            // find the beginning of the capture range
+            while (prevTokenPos>0 && data[prevTokenPos].kind()!=PToken::Kind::BeginCapture)
+            {
+              prevTokenPos--;
+            }
+          }
           data.insert(data.begin()+prevTokenPos,
                       c=='?' ? PToken(PToken::Kind::Optional) : PToken(PToken::Kind::Star));
           tokenPos++;
@@ -430,14 +439,15 @@ void Ex::Private::dump()
 
 /** Internal matching routine.
  *  @param tokenPos Offset into the token stream.
+ *  @param tokenLen The length of the token stream.
  *  @param str      The input string to match against.
  *  @param match    The object used to store the matching results.
  *  @param pos      The position in the input string to start with matching
  *  @param level    Recursion level (used for debugging)
  */
-bool Ex::Private::matchAt(size_t tokenPos,const std::string &str,Match &match,const size_t pos,int level) const
+bool Ex::Private::matchAt(size_t tokenPos,size_t tokenLen,std::string_view str,Match &match,const size_t pos,int level) const
 {
-  DBG("%d:matchAt(tokenPos=%zu, str='%s', pos=%zu)\n",level,tokenPos,str.c_str(),pos);
+  DBG("%d:matchAt(tokenPos=%zu, str='%s', pos=%zu)\n",level,tokenPos,pos<str.length() ? str.substr(pos).c_str() : "",pos);
   auto isStartIdChar = [](char c) { return isalpha(c) || c=='_'; };
   auto isIdChar      = [](char c) { return isalnum(c) || c=='_'; };
   auto matchCharClass = [this,isStartIdChar,isIdChar](size_t tp,char c) -> bool
@@ -473,41 +483,42 @@ bool Ex::Private::matchAt(size_t tokenPos,const std::string &str,Match &match,co
     return negate ? !found : found;
   };
   size_t index = pos;
-  enum SequenceType { Star, Optional };
-  auto processSequence = [this,&tokenPos,&index,&str,&matchCharClass,
+  enum SequenceType { Star, Optional, OptionalRange };
+  auto processSequence = [this,&tokenPos,&tokenLen,&index,&str,&matchCharClass,
                           &isStartIdChar,&isIdChar,&match,&level,&pos](SequenceType type) -> bool
   {
     size_t startIndex = index;
+    size_t len = str.length();
     PToken tok = data[++tokenPos];
     if (tok.kind()==PToken::Kind::Character) // 'x*' -> eat x's
     {
       char c_tok = tok.asciiValue();
-      while (index<=str.length() && str[index]==c_tok) { index++; if (type==Optional) break; }
+      while (index<len && str[index]==c_tok) { index++; if (type==Optional) break; }
       tokenPos++;
     }
     else if (tok.isCharClass()) // '[a-f0-4]* -> eat matching characters
     {
-      while (index<=str.length() && matchCharClass(tokenPos,str[index])) { index++; if (type==Optional) break; }
+      while (index<len && matchCharClass(tokenPos,str[index])) { index++; if (type==Optional) break; }
       tokenPos+=tok.value()+1; // skip over character ranges + end token
     }
     else if (tok.kind()==PToken::Kind::Alpha) // '\a*' -> eat start id characters
     {
-      while (index<=str.length() && isStartIdChar(str[index])) { index++; if (type==Optional) break; }
+      while (index<len && isStartIdChar(str[index])) { index++; if (type==Optional) break; }
       tokenPos++;
     }
     else if (tok.kind()==PToken::Kind::AlphaNum) // '\w*' -> eat id characters
     {
-      while (index<=str.length() && isIdChar(str[index])) { index++; if (type==Optional) break; }
+      while (index<len && isIdChar(str[index])) { index++; if (type==Optional) break; }
       tokenPos++;
     }
     else if (tok.kind()==PToken::Kind::WhiteSpace) // '\s*' -> eat spaces
     {
-      while (index<=str.length() && isspace(str[index])) { index++; if (type==Optional) break; }
+      while (index<len && isspace(str[index])) { index++; if (type==Optional) break; }
       tokenPos++;
     }
     else if (tok.kind()==PToken::Kind::Digit) // '\d*' -> eat digits
     {
-      while (index<=str.length() && isdigit(str[index])) { index++; if (type==Optional) break; }
+      while (index<len && isdigit(str[index])) { index++; if (type==Optional) break; }
       tokenPos++;
     }
     else if (tok.kind()==PToken::Kind::Any) // '.*' -> eat all
@@ -515,25 +526,39 @@ bool Ex::Private::matchAt(size_t tokenPos,const std::string &str,Match &match,co
       if (type==Optional) index++; else index = str.length();
       tokenPos++;
     }
+    else if (type==OptionalRange && tok.kind()==PToken::Kind::BeginCapture)
+    {
+      size_t tokenStart = ++tokenPos;
+      while (tokenPos<tokenLen && data[tokenPos].kind()!=PToken::Kind::EndCapture) { tokenPos++; }
+      Match rangeMatch;
+      rangeMatch.init(str);
+      bool found = matchAt(tokenStart,tokenPos,str,rangeMatch,index,level+1);
+      if (found)
+      {
+        index+=rangeMatch.length(); // (abc)? matches -> eat all
+      }
+      tokenPos++; // skip over EndCapture
+    }
     tokenPos++; // skip over end marker
-    while ((int)index>=(int)startIndex)
+    while (index>=startIndex)
     {
       // pattern 'x*xy' should match 'xy' and 'xxxxy'
-      bool found = matchAt(tokenPos,str,match,index,level+1);
+      bool found = matchAt(tokenPos,tokenLen,str,match,index,level+1);
       if (found)
       {
         match.setMatch(pos,index-pos+match.length());
         return true;
       }
+      if (index==0) break;
       index--;
     }
     return false;
   };
 
-  while (tokenPos<data.size())
+  while (tokenPos<tokenLen)
   {
     PToken tok = data[tokenPos];
-    //DBG("loop tokenPos=%zu token=%s\n",tokenPos,tok.kindStr());
+    DBG("loop tokenPos=%zu token=%s\n",tokenPos,tok.kindStr());
     if (tok.kind()==PToken::Kind::Character) // match literal character
     {
       char c_tok = tok.asciiValue();
@@ -603,7 +628,14 @@ bool Ex::Private::matchAt(size_t tokenPos,const std::string &str,Match &match,co
         case PToken::Kind::Star:
           return processSequence(Star);
         case PToken::Kind::Optional:
-          return processSequence(Optional);
+          if (tokenPos<tokenLen-1 && data[tokenPos+1].kind()==PToken::Kind::BeginCapture)
+          {
+            return processSequence(OptionalRange); // (...)?
+          }
+          else
+          {
+            return processSequence(Optional); // x?
+          }
         default:
           return false;
       }
@@ -614,13 +646,13 @@ bool Ex::Private::matchAt(size_t tokenPos,const std::string &str,Match &match,co
   return true;
 }
 
-static std::string wildcard2regex(const std::string &pattern)
+static std::string wildcard2regex(std::string_view pattern)
 {
   std::string result="^"; // match start of input
-  char c;
-  const char *p = pattern.c_str();
-  while ((c=*p++))
+  result.reserve(pattern.length());
+  for (size_t i=0;i<pattern.length();i++)
   {
+    char c=pattern[i];
     switch(c)
     {
       case '*':
@@ -639,10 +671,10 @@ static std::string wildcard2regex(const std::string &pattern)
          result+='\\'; result+=c;  // escape
          break;
       case '[':
-         if (*p=='^') // don't escape ^ after [
+         if (i<pattern.length()-1 && pattern[i+1]=='^') // don't escape ^ after [
          {
            result+="[^";
-           p++;
+           i++;
          }
          else
          {
@@ -659,7 +691,7 @@ static std::string wildcard2regex(const std::string &pattern)
 }
 
 
-Ex::Ex(const std::string &pattern, Mode mode)
+Ex::Ex(std::string_view pattern, Mode mode)
   : p(std::make_unique<Private>(mode==Mode::RegEx ? pattern : wildcard2regex(pattern)))
 {
   p->compile();
@@ -669,20 +701,18 @@ Ex::Ex(const std::string &pattern, Mode mode)
 #endif
 }
 
-Ex::~Ex()
-{
-}
+Ex::~Ex() = default;
 
-bool Ex::match(const std::string &str,Match &match,size_t pos) const
+bool Ex::match(std::string_view str,Match &match,size_t pos) const
 {
   bool found=false;
   if (p->data.size()==0 || p->error) return found;
-  match.init(&str);
+  match.init(str);
 
   PToken tok = p->data[0];
   if (tok.kind()==PToken::Kind::BeginOfLine) // only test match at the given position
   {
-    found = p->matchAt(0,str,match,pos,0);
+    found = p->matchAt(0,p->data.size(),str,match,pos,0);
   }
   else
   {
@@ -699,7 +729,7 @@ bool Ex::match(const std::string &str,Match &match,size_t pos) const
     }
     while (pos<str.length()) // search for a match starting at pos
     {
-      found = p->matchAt(0,str,match,pos,0);
+      found = p->matchAt(0,p->data.size(),str,match,pos,0);
       if (found) break;
       pos++;
     }
@@ -715,29 +745,29 @@ bool Ex::isValid() const
 
 //----------------------------------------------------------------------------------------
 
-bool search(const std::string &str,Match &match,const Ex &re,size_t pos)
+bool search(std::string_view str,Match &match,const Ex &re,size_t pos)
 {
   return re.match(str,match,pos);
 }
 
-bool search(const std::string &str,const Ex &re,size_t pos)
+bool search(std::string_view str,const Ex &re,size_t pos)
 {
   Match match;
   return re.match(str,match,pos);
 }
 
-bool match(const std::string &str,Match &match,const Ex &re)
+bool match(std::string_view str,Match &match,const Ex &re)
 {
   return re.match(str,match,0) && match.position()==0 && match.length()==str.length();
 }
 
-bool match(const std::string &str,const Ex &re)
+bool match(std::string_view str,const Ex &re)
 {
   Match match;
   return re.match(str,match,0) && match.position()==0 && match.length()==str.length();
 }
 
-std::string replace(const std::string &str,const Ex &re,const std::string &replacement)
+std::string replace(std::string_view str,const Ex &re,std::string_view replacement)
 {
   std::string result;
   Match match;
