@@ -32,6 +32,10 @@ class RTFCodeGenerator : public OutputCodeIntf
 
     OutputType type() const override { return OutputType::RTF; }
     void codify(const QCString &text) override;
+    void stripCodeComments(bool b) override;
+    void startSpecialComment() override;
+    void endSpecialComment() override;
+    void setStripIndentAmount(size_t amount) override;
     std::unique_ptr<OutputCodeIntf> clone() override { return std::make_unique<RTFCodeGenerator>(*this); }
     void writeCodeLink(CodeSymbolType type,
                        const QCString &ref,const QCString &file,
@@ -60,11 +64,14 @@ class RTFCodeGenerator : public OutputCodeIntf
     void setSourceFileName(const QCString &name);
     void setIndentLevel(int level) { m_indentLevel=level; }
     QCString rtf_Code_DepthStyle();
-    int  m_col = 0;
+    size_t  m_col = 0;
     TextStream *m_t;
     bool m_doxyCodeLineOpen = false;
     QCString m_sourceFileName;
     int m_indentLevel = 0;
+    bool m_stripCodeComments = false;
+    bool m_hide = false;
+    size_t m_stripIndentAmount = 0;
 };
 
 
@@ -135,7 +142,7 @@ class RTFGenerator : public OutputGenerator, public OutputGenIntf
     void endTextLink() override;
     void startTypewriter() override { m_t << "{\\f2 "; }
     void endTypewriter() override { m_t << "}";      }
-    void startGroupHeader(int) override;
+    void startGroupHeader(const QCString &,int) override;
     void endGroupHeader(int) override;
     void startItemListItem() override;
     void endItemListItem() override;
@@ -144,7 +151,7 @@ class RTFGenerator : public OutputGenerator, public OutputGenIntf
     void endMemberSections() override {}
     void startHeaderSection() override {}
     void endHeaderSection() override {}
-    void startMemberHeader(const QCString &,int) override { startGroupHeader(0); }
+    void startMemberHeader(const QCString &,int) override { startGroupHeader("",0); }
     void endMemberHeader() override { endGroupHeader(FALSE); }
     void startMemberSubtitle() override;
     void endMemberSubtitle() override;
@@ -209,11 +216,12 @@ class RTFGenerator : public OutputGenerator, public OutputGenIntf
     void endPageRef(const QCString &,const QCString &) override;
     void startQuickIndices() override {}
     void endQuickIndices() override {}
-    void writeSplitBar(const QCString &) override {}
+    void writeSplitBar(const QCString &,const QCString &) override {}
     void writeNavigationPath(const QCString &) override {}
     void writeLogo() override {}
-    void writeQuickLinks(HighlightedItem,const QCString &) override {}
+    void writeQuickLinks(HighlightedItem,const QCString &,bool) override {}
     void writeSummaryLink(const QCString &,const QCString &,const QCString &,bool) override {}
+    void writePageOutline() override {}
     void startContents() override {}
     void endContents() override {}
     void writeNonBreakableSpace(int) override;
@@ -241,7 +249,7 @@ class RTFGenerator : public OutputGenerator, public OutputGenIntf
     void endDirDepGraph(DotDirDeps &g) override;
     void writeGraphicalHierarchy(DotGfxHierarchyTable &) override {}
 
-    void startMemberGroupHeader(bool) override;
+    void startMemberGroupHeader(const QCString &,bool) override;
     void endMemberGroupHeader() override;
     void startMemberGroupDocs() override;
     void endMemberGroupDocs() override;
@@ -290,7 +298,10 @@ class RTFGenerator : public OutputGenerator, public OutputGenIntf
     void writeLabel(const QCString &l,bool isLast) override;
     void endLabels() override;
 
-    void writeLocalToc(const SectionRefs &,const LocalToc &) override {}
+    void startLocalToc(int) override {}
+    void endLocalToc() override {}
+    void startTocEntry(const SectionInfo *) override {}
+    void endTocEntry(const SectionInfo *) override {}
 
     void startPlainFile(const QCString &name) override { OutputGenerator::startPlainFile(name); }
     void endPlainFile() override { OutputGenerator::endPlainFile(); }
