@@ -59,6 +59,12 @@
 // debug inside output
 //#define XML_DB(x) QCString __t;__t.sprintf x;m_t << __t
 
+static void writeXMLDocBlock(TextStream &t,
+                      const QCString &fileName,
+                      int lineNr,
+                      const Definition *scope,
+                      const MemberDef * md,
+                      const QCString &text);
 //------------------
 
 inline void writeXMLString(TextStream &t,const QCString &s)
@@ -132,7 +138,7 @@ static void writeCombineScript()
   std::ofstream t = Portable::openOutputStream(fileName);
   if (!t.is_open())
   {
-    err("Cannot open file %s for writing!\n",qPrint(fileName));
+    err("Cannot open file {} for writing!\n",fileName);
     return;
   }
 
@@ -403,6 +409,20 @@ static void writeTemplateArgumentList(TextStream &t,
         linkifyText(TextGeneratorXMLImpl(t),scope,fileScope,nullptr,a.typeConstraint);
         t << "</typeconstraint>\n";
       }
+      if (a.hasTemplateDocumentation())
+      {
+        t << indentStr << "    <briefdescription>\n";
+        t << indentStr << "      ";
+        if (scope)
+        {
+          writeXMLDocBlock(t,scope->briefFile(),scope->briefLine(),scope,nullptr,a.docs);
+        }
+        else
+        {
+          writeXMLDocBlock(t,fileScope->briefFile(),fileScope->briefLine(),fileScope,nullptr,a.docs);
+        }
+        t << indentStr << "    </briefdescription>\n";
+      }
       t << indentStr << "  </param>\n";
     }
     t << indentStr << "</templateparamlist>\n";
@@ -437,7 +457,7 @@ static void writeXMLDocBlock(TextStream &t,
   auto parser { createDocParser() };
   auto ast    { validatingParseDoc(*parser.get(),
                                    fileName,lineNr,scope,md,text,FALSE,FALSE,
-                                   QCString(),FALSE,FALSE,Config_getBool(MARKDOWN_SUPPORT)) };
+                                   QCString(),FALSE,FALSE) };
   auto astImpl = dynamic_cast<const DocNodeAST*>(ast.get());
   if (astImpl)
   {
@@ -563,6 +583,54 @@ static QCString memberOutputFileBase(const MemberDef *md)
   return md->getOutputFileBase();
 }
 
+// Removes a keyword from a given string
+// @param str string from which to strip the keyword
+// @param needSpace true if spacing is required around the keyword
+// @return true if the keyword was removed, false otherwise
+static bool stripKeyword(QCString& str, const char *keyword, bool needSpace)
+{
+  bool found      = false;
+  int searchStart = 0;
+  int len         = static_cast<int>(strlen(keyword));
+  int searchEnd   = static_cast<int>(str.size());
+  while (searchStart<searchEnd)
+  {
+    int index = str.find(keyword, searchStart);
+    if (index==-1)
+    {
+      break; // no more occurrences found
+    }
+    int end = index + len;
+    if (needSpace)
+    {
+      if ((index>0        && str[index-1]!=' ') ||  // at the start of the string or preceded by a space, or
+          (end!=searchEnd && str[end]    !=' ')     // at the end of the string or followed by a space.
+         )
+      {
+        searchStart = end;
+        continue; // no a standalone word
+      }
+    }
+    if (needSpace && index>0) // strip with space before keyword
+    {
+      str.remove(index-1, len+1);
+      searchEnd -= (len+1);
+    }
+    else if (needSpace && end<searchEnd) // strip with space after string starting with keyword
+    {
+      str.remove(index, len+1);
+      searchEnd -= (len+1);
+    }
+    else // strip just keyword
+    {
+      str.remove(index, len);
+      searchEnd -= len;
+    }
+    found = true;
+  }
+  return found;
+}
+
 static QCString extractNoExcept(QCString &argsStr)
 {
   QCString expr;
@@ -669,6 +737,18 @@ static void generateXMLForMember(const MemberDef *md,TextStream &ti,TextStream &
     {
       typeStr=argsStr.mid(i+2).stripWhiteSpace();
       argsStr=argsStr.left(i).stripWhiteSpace();
+      if (stripKeyword(typeStr, "override", true))
+      {
+        argsStr += " override";
+      }
+      if (stripKeyword(typeStr, "final", true))
+      {
+        argsStr += " final";
+      }
+      if (stripKeyword(typeStr, "=0", false))
+      {
+        argsStr += "=0";
+      }
       i=defStr.find("auto ");
       if (i!=-1)
       {
@@ -701,14 +781,7 @@ static void generateXMLForMember(const MemberDef *md,TextStream &ti,TextStream &
   }
   //enum { define_t,variable_t,typedef_t,enum_t,function_t } xmlType = function_t;
 
-  t << "\" prot=\"";
-  switch (md->protection())
-  {
-    case Protection::Public:    t << "public";     break;
-    case Protection::Protected: t << "protected";  break;
-    case Protection::Private:   t << "private";    break;
-    case Protection::Package:   t << "package";    break;
-  }
+  t << "\" prot=\"" << to_string_lower(md->protection());
   t << "\"";
 
   t << " static=\"";
@@ -802,14 +875,7 @@ static void generateXMLForMember(const MemberDef *md,TextStream &ti,TextStream &
       t << " volatile=\"yes\"";
     }
 
-    t << " virt=\"";
-    switch (md->virtualness())
-    {
-      case Specifier::Normal:  t << "non-virtual";  break;
-      case Specifier::Virtual: t << "virtual";      break;
-      case Specifier::Pure:    t << "pure-virtual"; break;
-      default: ASSERT(0);
-    }
+    t << " virt=\"" << to_string_lower(md->virtualness());
     t << "\"";
   }
 
@@ -1128,14 +1194,7 @@ static void generateXMLForMember(const MemberDef *md,TextStream &ti,TextStream &
          << convertToXML(emd->name()) << "</name></member>\n";
 
       t << "        <enumvalue id=\"" << memberOutputFileBase(md) << "_1"
-        << emd->anchor() << "\" prot=\"";
-      switch (emd->protection())
-      {
-        case Protection::Public:    t << "public";    break;
-        case Protection::Protected: t << "protected"; break;
-        case Protection::Private:   t << "private";   break;
-        case Protection::Package:   t << "package";   break;
-      }
+        << emd->anchor() << "\" prot=\"" << to_string_lower(emd->protection());
       t << "\">\n";
       t << "          <name>";
       writeXMLString(t,emd->name());
@@ -1263,22 +1322,8 @@ static void writeListOfAllMembers(const ClassDef *cd,TextStream &t)
         Protection prot = mi->prot();
         Specifier virt=md->virtualness();
         t << "      <member refid=\"" << memberOutputFileBase(md) << "_1" <<
-          md->anchor() << "\" prot=\"";
-        switch (prot)
-        {
-          case Protection::Public:    t << "public";    break;
-          case Protection::Protected: t << "protected"; break;
-          case Protection::Private:   t << "private";   break;
-          case Protection::Package:   t << "package";   break;
-        }
-        t << "\" virt=\"";
-        switch(virt)
-        {
-          case Specifier::Normal:  t << "non-virtual";  break;
-          case Specifier::Virtual: t << "virtual";      break;
-          case Specifier::Pure:    t << "pure-virtual"; break;
-        }
-        t << "\"";
+          md->anchor() << "\" prot=\"" << to_string_lower(prot);
+        t << "\" virt=\"" << to_string_lower(virt) << "\"";
         if (!mi->ambiguityResolutionScope().isEmpty())
         {
           t << " ambiguityscope=\"" << convertToXML(mi->ambiguityResolutionScope()) << "\"";
@@ -1298,14 +1343,7 @@ static void writeInnerClasses(const ClassLinkedRefMap &cl,TextStream &t)
     if (!cd->isHidden() && !cd->isAnonymous())
     {
       t << "    <innerclass refid=\"" << classOutputFileBase(cd)
-        << "\" prot=\"";
-      switch(cd->protection())
-      {
-        case Protection::Public:    t << "public";     break;
-        case Protection::Protected: t << "protected";  break;
-        case Protection::Private:   t << "private";    break;
-        case Protection::Package:   t << "package";    break;
-      }
+        << "\" prot=\"" << to_string_lower(cd->protection());
       t << "\">" << convertToXML(cd->name()) << "</innerclass>\n";
     }
   }
@@ -1458,7 +1496,7 @@ static void generateXMLForClass(const ClassDef *cd,TextStream &ti)
   if (cd->isImplicitTemplateInstance())  return; // skip generated template instances.
   if (cd->isArtificial())       return; // skip artificially created classes
 
-  msg("Generating XML output for class %s\n",qPrint(cd->name()));
+  msg("Generating XML output for class {}\n",cd->name());
 
   ti << "  <compound refid=\"" << classOutputFileBase(cd)
      << "\" kind=\"" << cd->compoundTypeString()
@@ -1469,7 +1507,7 @@ static void generateXMLForClass(const ClassDef *cd,TextStream &ti)
   std::ofstream f = Portable::openOutputStream(fileName);
   if (!f.is_open())
   {
-    err("Cannot open file %s for writing!\n",qPrint(fileName));
+    err("Cannot open file {} for writing!\n",fileName);
     return;
   }
   TextStream t(&f);
@@ -1479,13 +1517,7 @@ static void generateXMLForClass(const ClassDef *cd,TextStream &ti)
     << classOutputFileBase(cd) << "\" kind=\""
     << cd->compoundTypeString() << "\" language=\""
     << langToString(cd->getLanguage()) << "\" prot=\"";
-  switch (cd->protection())
-  {
-    case Protection::Public:    t << "public";    break;
-    case Protection::Protected: t << "protected"; break;
-    case Protection::Private:   t << "private";   break;
-    case Protection::Package:   t << "package";   break;
-  }
+  t << to_string_lower(cd->protection());
   if (cd->isFinal()) t << "\" final=\"yes";
   if (cd->isSealed()) t << "\" sealed=\"yes";
   if (cd->isAbstract()) t << "\" abstract=\"yes";
@@ -1502,21 +1534,11 @@ static void generateXMLForClass(const ClassDef *cd,TextStream &ti)
     {
       t << "refid=\"" << classOutputFileBase(bcd.classDef) << "\" ";
     }
+    if (bcd.prot == Protection::Package) ASSERT(0);
     t << "prot=\"";
-    switch (bcd.prot)
-    {
-      case Protection::Public:    t << "public";    break;
-      case Protection::Protected: t << "protected"; break;
-      case Protection::Private:   t << "private";   break;
-      case Protection::Package: ASSERT(0); break;
-    }
+    t << to_string_lower(bcd.prot);
     t << "\" virt=\"";
-    switch(bcd.virt)
-    {
-      case Specifier::Normal:  t << "non-virtual";  break;
-      case Specifier::Virtual: t << "virtual";      break;
-      case Specifier::Pure:    t <<"pure-virtual"; break;
-    }
+    t << to_string_lower(bcd.virt);
     t << "\">";
     if (!bcd.templSpecifiers.isEmpty())
     {
@@ -1533,23 +1555,13 @@ static void generateXMLForClass(const ClassDef *cd,TextStream &ti)
   }
   for (const auto &bcd : cd->subClasses())
   {
+    if (bcd.prot == Protection::Package) ASSERT(0);
     t << "    <derivedcompoundref refid=\""
       << classOutputFileBase(bcd.classDef)
       << "\" prot=\"";
-    switch (bcd.prot)
-    {
-      case Protection::Public:    t << "public";    break;
-      case Protection::Protected: t << "protected"; break;
-      case Protection::Private:   t << "private";   break;
-      case Protection::Package: ASSERT(0); break;
-    }
+    t << to_string_lower(bcd.prot);
     t << "\" virt=\"";
-    switch (bcd.virt)
-    {
-      case Specifier::Normal:  t << "non-virtual";  break;
-      case Specifier::Virtual: t << "virtual";      break;
-      case Specifier::Pure:    t << "pure-virtual"; break;
-    }
+    t << to_string_lower(bcd.virt);
     t << "\">" << convertToXML(bcd.classDef->displayName())
       << "</derivedcompoundref>\n";
   }
@@ -1640,7 +1652,7 @@ static void generateXMLForConcept(const ConceptDef *cd,TextStream &ti)
   std::ofstream f = Portable::openOutputStream(fileName);
   if (!f.is_open())
   {
-    err("Cannot open file %s for writing!\n",qPrint(fileName));
+    err("Cannot open file {} for writing!\n",fileName);
     return;
   }
   TextStream t(&f);
@@ -1685,7 +1697,7 @@ static void generateXMLForModule(const ModuleDef *mod,TextStream &ti)
   std::ofstream f = Portable::openOutputStream(fileName);
   if (!f.is_open())
   {
-    err("Cannot open file %s for writing!\n",qPrint(fileName));
+    err("Cannot open file {} for writing!\n",fileName);
     return;
   }
   TextStream t(&f);
@@ -1750,7 +1762,7 @@ static void generateXMLForNamespace(const NamespaceDef *nd,TextStream &ti)
   std::ofstream f = Portable::openOutputStream(fileName);
   if (!f.is_open())
   {
-    err("Cannot open file %s for writing!\n",qPrint(fileName));
+    err("Cannot open file {} for writing!\n",fileName);
     return;
   }
   TextStream t(&f);
@@ -1828,7 +1840,7 @@ static void generateXMLForFile(FileDef *fd,TextStream &ti)
   std::ofstream f = Portable::openOutputStream(fileName);
   if (!f.is_open())
   {
-    err("Cannot open file %s for writing!\n",qPrint(fileName));
+    err("Cannot open file {} for writing!\n",fileName);
     return;
   }
   TextStream t(&f);
@@ -1849,7 +1861,7 @@ static void generateXMLForFile(FileDef *fd,TextStream &ti)
       t << " refid=\"" << inc.fileDef->getOutputFileBase() << "\"";
     }
     t << " local=\"" << ((inc.kind & IncludeKind_LocalMask) ? "yes" : "no") << "\">";
-    t << inc.includeName;
+    t << convertToXML(inc.includeName);
     t << "</includes>\n";
   }
 
@@ -1861,7 +1873,7 @@ static void generateXMLForFile(FileDef *fd,TextStream &ti)
       t << " refid=\"" << inc.fileDef->getOutputFileBase() << "\"";
     }
     t << " local=\"" << ((inc.kind &IncludeKind_LocalMask) ? "yes" : "no") << "\">";
-    t << inc.includeName;
+    t << convertToXML(inc.includeName);
     t << "</includedby>\n";
   }
 
@@ -1940,7 +1952,7 @@ static void generateXMLForGroup(const GroupDef *gd,TextStream &ti)
   std::ofstream f = Portable::openOutputStream(fileName);
   if (!f.is_open())
   {
-    err("Cannot open file %s for writing!\n",qPrint(fileName));
+    err("Cannot open file {} for writing!\n",fileName);
     return;
   }
   TextStream t(&f);
@@ -1997,7 +2009,7 @@ static void generateXMLForDir(DirDef *dd,TextStream &ti)
   std::ofstream f = Portable::openOutputStream(fileName);
   if (!f.is_open())
   {
-    err("Cannot open file %s for writing!\n",qPrint(fileName));
+    err("Cannot open file {} for writing!\n",fileName);
     return;
   }
   TextStream t(&f);
@@ -2050,7 +2062,7 @@ static void generateXMLForPage(PageDef *pd,TextStream &ti,bool isExample)
   std::ofstream f = Portable::openOutputStream(fileName);
   if (!f.is_open())
   {
-    err("Cannot open file %s for writing!\n",qPrint(fileName));
+    err("Cannot open file {} for writing!\n",fileName);
     return;
   }
   TextStream t(&f);
@@ -2133,7 +2145,13 @@ static void generateXMLForPage(PageDef *pd,TextStream &ti,bool isExample)
           QCString label = convertToXML(si->label());
           if (titleDoc.isEmpty()) titleDoc = label;
           incIndent("<tocsect>");
-          writeIndent(); t << "<name>" << titleDoc << "</name>\n";
+          writeIndent(); t << "<name>" << titleDoc << "</name>\n"; // kept for backwards compatibility
+          writeIndent(); t << "<docs>";
+          if (!si->title().isEmpty())
+          {
+            writeXMLDocBlock(t,pd->docFile(),pd->docLine(),pd,nullptr,si->title());
+          }
+          t << "</docs>\n";
           writeIndent(); t << "<reference>"  <<  convertToXML(pageName) << "_1" << label << "</reference>\n";
           inLi[nextLevel]=true;
           level = nextLevel;
@@ -2196,7 +2214,7 @@ void generateXML()
   std::ofstream f = Portable::openOutputStream(fileName);
   if (!f.is_open())
   {
-    err("Cannot open file %s for writing!\n",qPrint(fileName));
+    err("Cannot open file {} for writing!\n",fileName);
     return;
   }
   {
@@ -2232,7 +2250,7 @@ void generateXML()
   f = Portable::openOutputStream(fileName);
   if (!f.is_open())
   {
-    err("Cannot open file %s for writing!\n",qPrint(fileName));
+    err("Cannot open file {} for writing!\n",fileName);
     return;
   }
   {
@@ -2268,7 +2286,7 @@ void generateXML()
   f = Portable::openOutputStream(fileName);
   if (!f.is_open())
   {
-    err("Cannot open file %s for writing\n",fileName.data());
+    err("Cannot open file {} for writing\n",fileName);
     return;
   }
   else
@@ -2282,7 +2300,7 @@ void generateXML()
   f = Portable::openOutputStream(fileName);
   if (!f.is_open())
   {
-    err("Cannot open file %s for writing!\n",qPrint(fileName));
+    err("Cannot open file {} for writing!\n",fileName);
     return;
   }
   else
@@ -2303,45 +2321,45 @@ void generateXML()
     }
     for (const auto &cd : *Doxygen::conceptLinkedMap)
     {
-      msg("Generating XML output for concept %s\n",qPrint(cd->displayName()));
+      msg("Generating XML output for concept {}\n",cd->displayName());
       generateXMLForConcept(cd.get(),t);
     }
     for (const auto &nd : *Doxygen::namespaceLinkedMap)
     {
-      msg("Generating XML output for namespace %s\n",qPrint(nd->displayName()));
+      msg("Generating XML output for namespace {}\n",nd->displayName());
       generateXMLForNamespace(nd.get(),t);
     }
     for (const auto &fn : *Doxygen::inputNameLinkedMap)
     {
       for (const auto &fd : *fn)
       {
-        msg("Generating XML output for file %s\n",qPrint(fd->name()));
+        msg("Generating XML output for file {}\n",fd->name());
         generateXMLForFile(fd.get(),t);
       }
     }
     for (const auto &gd : *Doxygen::groupLinkedMap)
     {
-      msg("Generating XML output for group %s\n",qPrint(gd->name()));
+      msg("Generating XML output for group {}\n",gd->name());
       generateXMLForGroup(gd.get(),t);
     }
     for (const auto &pd : *Doxygen::pageLinkedMap)
     {
-      msg("Generating XML output for page %s\n",qPrint(pd->name()));
+      msg("Generating XML output for page {}\n",pd->name());
       generateXMLForPage(pd.get(),t,FALSE);
     }
     for (const auto &dd : *Doxygen::dirLinkedMap)
     {
-      msg("Generate XML output for dir %s\n",qPrint(dd->name()));
+      msg("Generate XML output for dir {}\n",dd->name());
       generateXMLForDir(dd.get(),t);
     }
     for (const auto &mod : ModuleManager::instance().modules())
     {
-      msg("Generating XML output for module %s\n",qPrint(mod->name()));
+      msg("Generating XML output for module {}\n",mod->name());
       generateXMLForModule(mod.get(),t);
     }
     for (const auto &pd : *Doxygen::exampleLinkedMap)
     {
-      msg("Generating XML output for example %s\n",qPrint(pd->name()));
+      msg("Generating XML output for example {}\n",pd->name());
       generateXMLForPage(pd.get(),t,TRUE);
     }
     if (Doxygen::mainPage)
